# -*- coding: utf-8 -*-

import pandas as pd
from futu.common.open_context_base import OpenContextBase
from futu.trade.trade_query import *
from futu.common.err import *

class OpenTradeContextBase(OpenContextBase):
    """Class for set context of HK stock trade"""

    def __init__(self, trd_mkt, host="127.0.0.1", port=11111, is_encrypt=None, security_firm=SecurityFirm.FUTUSECURITIES):
        self.__trd_mkt = trd_mkt
        self._ctx_unlock = None
        self.__last_acc_list = []
        self.__is_acc_sub_push = False
        self.__security_firm = security_firm

        # if host != "127.0.0.1" and host != "localhost" and is_encrypt is None:
        #     '''非本地连接必须加密，以免远程攻击'''
        #     print("{} is not local connection!".format(host))
        #     raise Exception('Non-local connections must be encrypted')

        super(OpenTradeContextBase, self).__init__(host, port, False, is_encrypt=is_encrypt)

    def close(self):
        """
        to call close old obj before loop create new, otherwise socket will encounter erro 10053 or more!
        """
        super(OpenTradeContextBase, self).close()

    def on_api_socket_reconnected(self):
        """for API socket reconnected"""
        self.__is_acc_sub_push = False
        self.__last_acc_list = []

        ret, msg = RET_OK, ''
        # auto unlock trade
        if self._ctx_unlock is not None:
            password, password_md5 = self._ctx_unlock
            ret, data = self.unlock_trade(password, password_md5)
            logger.debug('auto unlock trade ret={},data={}'.format(ret, data))
            if ret != RET_OK:
                msg = data

        # 定阅交易帐号推送
        if ret == RET_OK:
            self.__check_acc_sub_push()

        return ret, msg

    def get_acc_list(self):
        """
        :return: (ret, data)
        """
        query_processor = self._get_sync_query_processor(
            GetAccountList.pack_req, GetAccountList.unpack_rsp)

        kargs = {
            'user_id': self.get_login_user_id(),
            'conn_id': self.get_sync_conn_id()
        }

        ret_code, msg, acc_list = query_processor(**kargs)
        if ret_code != RET_OK:
            return RET_ERROR, msg

        # 记录当前市场的帐号列表
        self.__last_acc_list = []

        for record in acc_list:
            trdMkt_list = record["trdMarket_list"]
            if self.__trd_mkt in trdMkt_list:
                if record['trd_env'] == TrdEnv.SIMULATE or record['security_firm'] == NoneDataValue  or record['security_firm'] == self.__security_firm:
                    self.__last_acc_list.append({
                        "trd_env": record["trd_env"],
                        "acc_id": record["acc_id"],
                        "acc_type": record["acc_type"],
                        "card_num": record["card_num"],
                        "security_firm": record["security_firm"]})

        col_list = ["acc_id", "trd_env", "acc_type", "card_num", "security_firm"]

        acc_table = pd.DataFrame(copy(self.__last_acc_list), columns=col_list)

        return RET_OK, acc_table

    def unlock_trade(self, password=None, password_md5=None, is_unlock=True):
        """
        交易解锁，安全考虑，所有的交易api,需成功解锁后才可操作
        :param password: 明文密码字符串 (二选一）
        :param password_md5: 密码的md5字符串（二选一）
        :param is_unlock: 解锁 = True, 锁定 = False
        :return:(ret, data) ret == RET_OK时, data为None，如果之前已经解锁过了，data为提示字符串，指示出已经解锁
                            ret != RET_OK时， data为错误字符串
        """

        # 仅支持真实交易的市场可以解锁，模拟交易不需要解锁
        md5_val = ''
        if is_unlock:
            ret = TRADE.check_mkt_envtype(self.__trd_mkt, TrdEnv.REAL)
            if not ret:
                return RET_OK, Err.NoNeedUnlock.text

            if password is None and password_md5 is None:
                return RET_ERROR, make_msg(Err.ParamErr, password=password, password_md5=password_md5)

            md5_val = str(password_md5) if password_md5 else md5_transform(str(password))

        # 解锁要求先拉一次帐户列表, 目前仅真实环境需要解锁
        ret, msg, acc_id = self._check_acc_id(TrdEnv.REAL, 0)
        if ret != RET_OK:
            return ret, msg

        query_processor = self._get_sync_query_processor(
                UnlockTrade.pack_req, UnlockTrade.unpack_rsp)

        kargs = {
            'is_unlock': is_unlock,
            'password_md5': md5_val,
            'conn_id': self.get_sync_conn_id()
        }

        ret_code, msg, _ = query_processor(**kargs)
        if ret_code != RET_OK:
            return RET_ERROR, msg

        # reconnected to auto unlock
        if RET_OK == ret_code:
            self._ctx_unlock = (password, password_md5) if is_unlock else None

        # 定阅交易帐号推送
        if is_unlock and ret_code == RET_OK:
            self.__check_acc_sub_push()

        if msg is not None and len(msg) > 0:
            return RET_OK, msg
        return RET_OK, None

    def _async_sub_acc_push(self, acc_id_list):
        """
        异步连接指定要接收送的acc id
        :param acc_id:
        :return:
        """
        kargs = {
            'acc_id_list': acc_id_list,
            'conn_id': self.get_async_conn_id(),
        }
        ret_code, msg, push_req_str = SubAccPush.pack_req(**kargs)
        if ret_code == RET_OK:
            self._send_async_req(push_req_str)

        return RET_OK, None

    def on_async_sub_acc_push(self, ret_code, msg):
        self.__is_acc_sub_push = ret_code == RET_OK
        if not self.__is_acc_sub_push:
            logger.error("ret={} msg={}".format(ret_code, msg))

    def _check_trd_env(self, trd_env):
        is_enable = TRADE.check_mkt_envtype(self.__trd_mkt, trd_env)
        if not is_enable:
            return RET_ERROR, ERROR_STR_PREFIX + "the type of environment param is wrong "

        return RET_OK, ""

    def __check_acc_sub_push(self):
        if self.__is_acc_sub_push:
            return

        if len(self.__last_acc_list) == 0:
            ret, _ = self.get_acc_list()
            if ret != RET_OK:
                return

        acc_id_list = [x['acc_id'] for x in self.__last_acc_list]

        if len(acc_id_list):
            self._async_sub_acc_push(acc_id_list)

    def _check_acc_id(self, trd_env, acc_id):
        if acc_id == 0:
            if len(self.__last_acc_list) == 0:
                ret, content = self.get_acc_list()
                if ret != RET_OK:
                    return ret, content, acc_id
            acc_id = self._get_default_acc_id(trd_env)

        msg = "" if acc_id != 0 else ERROR_STR_PREFIX + "No one available account!"
        ret = RET_OK if acc_id != 0 else RET_ERROR

        return ret, msg, acc_id

    def _check_order_status(self, status_filter_list):
        unique_and_normalize_list(status_filter_list)
        for status in status_filter_list:
            if not OrderStatus.if_has_key(status):
                return RET_ERROR, ERROR_STR_PREFIX + "the type of status_filter_list param is wrong "
        return RET_OK, "",

    def _get_default_acc_id(self, trd_env):
        for record in self.__last_acc_list:
            if  record['trd_env'] == trd_env:
                return record['acc_id']
        return 0

    def _get_default_acc_id(self, trd_env):
        for record in self.__last_acc_list:
            if record['trd_env'] == trd_env:
                return record['acc_id']
        return 0

    def _get_acc_id_by_acc_index(self, trd_env, acc_index=0):
        ret, msg = self.get_acc_list()
        if ret != RET_OK:
            return ret, msg, None
        acc_table = msg
        env_list = [trd_env]
        acc_table = acc_table[acc_table['trd_env'].isin(env_list)]
        acc_table = acc_table.reset_index(drop=True)

        total_acc_num = acc_table.shape[0]
        if total_acc_num == 0:
            msg = Err.NoAccForSecurityFirm.text
            return RET_ERROR, msg, acc_index
        elif acc_index >= total_acc_num:
            msg = ERROR_STR_PREFIX + "the index {0} is out of the total amount {1} ".format(acc_index, total_acc_num)
            return RET_ERROR, msg, acc_index
        return RET_OK, "", acc_table['acc_id'][acc_index]

    def _check_acc_id_exist(self, trd_env, acc_id):
        ret, msg = self.get_acc_list()
        if ret != RET_OK:
            return ret, msg, acc_id
        content = msg

        acc_index = content[(content.acc_id == acc_id) & (content.trd_env == trd_env)].index.tolist()
        if len(acc_index):
            return RET_OK, "", acc_id
        else:
            return RET_ERROR, ERROR_STR_PREFIX + "This account is not available account!", acc_id

    def _check_acc_id_and_acc_index(self, trd_env, acc_id, acc_index):
        if acc_id == 0:
            ret, msg, acc_id = self._get_acc_id_by_acc_index(trd_env, acc_index)
            if ret != RET_OK:
                return ret, msg, acc_id
        else:
            ret, msg, acc_id = self._check_acc_id_exist(trd_env, acc_id)
            if ret != RET_OK:
                return ret, msg, acc_id
        return RET_OK, "", acc_id

    def accinfo_query(self, trd_env=TrdEnv.REAL, acc_id=0, acc_index=0, refresh_cache=False, currency=Currency.HKD):
        """
        :param trd_env:
        :param acc_id:
        :param acc_index:
        :return:
        """
        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg

        ret, msg, acc_id = self._check_acc_id_and_acc_index(trd_env, acc_id, acc_index)
        if ret != RET_OK:
            return ret, msg

        query_processor = self._get_sync_query_processor(
            AccInfoQuery.pack_req, AccInfoQuery.unpack_rsp)

        kargs = {
            'acc_id': int(acc_id),
            'trd_env': trd_env,
            'trd_market': self.__trd_mkt,
            'conn_id': self.get_sync_conn_id(),
            'refresh_cache': refresh_cache,
            'currency': currency
        }

        ret_code, msg, accinfo_list = query_processor(**kargs)
        if ret_code != RET_OK:
            return RET_ERROR, msg

        col_list = [
            'power', 'total_assets', 'cash', 'market_val', 'frozen_cash', 'avl_withdrawal_cash', 'currency',
            'available_funds', 'unrealized_pl', 'realized_pl', 'risk_level', 'initial_margin', 'maintenance_margin',
            'hk_cash', 'hk_avl_withdrawal_cash', 'us_cash', 'us_avl_withdrawal_cash'
        ]
        accinfo_frame_table = pd.DataFrame(accinfo_list, columns=col_list)

        return RET_OK, accinfo_frame_table

    def _check_stock_code(self, code):
        stock_code = ''
        if code is not None and code != '':
            ret_code, content = split_stock_str(str(code))
            if ret_code == RET_OK:
                _, stock_code = content
            else:
                stock_code = code
        return RET_OK, "", stock_code

    def _split_stock_code(self, code):
        stock_str = str(code)

        split_loc = stock_str.find(".")
        '''do not use the built-in split function in python.
        The built-in function cannot handle some stock strings correctly.
        for instance, US..DJI, where the dot . itself is a part of original code'''
        if 0 <= split_loc < len(stock_str) - 1 and Market.if_has_key(stock_str[0:split_loc]):
            market_str = stock_str[0:split_loc]
            partial_stock_str = stock_str[split_loc + 1:]
            return RET_OK, (market_str, partial_stock_str)

        else:
            error_str = ERROR_STR_PREFIX + "format of %s is wrong. (US.AAPL, HK.00700, SZ.000001)" % stock_str
            return RET_ERROR, error_str

    def position_list_query(self, code='', pl_ratio_min=None,
                            pl_ratio_max=None, trd_env=TrdEnv.REAL, acc_id=0, acc_index=0, refresh_cache=False):
        """for querying the position list"""
        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg

        ret, msg, acc_id = self._check_acc_id_and_acc_index(trd_env, acc_id, acc_index)
        if ret != RET_OK:
            return ret, msg

        ret, msg, stock_code = self._check_stock_code(code)
        if ret != RET_OK:
            return ret, msg

        query_processor = self._get_sync_query_processor(
            PositionListQuery.pack_req, PositionListQuery.unpack_rsp)

        kargs = {
            'code': str(stock_code),
            'pl_ratio_min': pl_ratio_min,
            'pl_ratio_max': pl_ratio_max,
            'trd_mkt': self.__trd_mkt,
            'trd_env': trd_env,
            'acc_id': acc_id,
            'conn_id': self.get_sync_conn_id(),
            'refresh_cache': refresh_cache
        }
        ret_code, msg, position_list = query_processor(**kargs)

        if ret_code != RET_OK:
            return RET_ERROR, msg

        col_list = [
            "code", "stock_name", "qty", "can_sell_qty", "cost_price",
            "cost_price_valid", "market_val", "nominal_price", "pl_ratio",
            "pl_ratio_valid", "pl_val", "pl_val_valid", "today_buy_qty",
            "today_buy_val", "today_pl_val", "today_sell_qty", "today_sell_val",
            "position_side", "unrealized_pl", "realized_pl"
        ]

        position_list_table = pd.DataFrame(position_list, columns=col_list)
        return RET_OK, position_list_table

    def order_list_query(self, order_id="", status_filter_list=[], code='', start='', end='',
                         trd_env=TrdEnv.REAL, acc_id=0, acc_index=0, refresh_cache=False):

        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg

        ret, msg, acc_id = self._check_acc_id_and_acc_index(trd_env, acc_id, acc_index)
        if ret != RET_OK:
            return ret, msg

        ret_code, ret_data = self._order_list_query_impl(order_id, status_filter_list,
                                                         code, start, end, trd_env, acc_id,
                                                         refresh_cache)
        if ret_code != RET_OK:
            return ret_code, ret_data

        col_list = [
            "code", "stock_name", "trd_side", "order_type", "order_status",
            "order_id", "qty", "price", "create_time", "updated_time",
            "dealt_qty", "dealt_avg_price", "last_err_msg", "remark",
            "time_in_force", "fill_outside_rth"

        ]
        order_list = ret_data
        order_list_table = pd.DataFrame(order_list, columns=col_list)

        return RET_OK, order_list_table

    def _order_list_query_impl(self, order_id, status_filter_list, code, start, end, trd_env, acc_id, refresh_cache):
        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg
        ret, msg , acc_id = self._check_acc_id(trd_env, acc_id)
        if ret != RET_OK:
            return ret, msg

        ret, msg, stock_code = self._check_stock_code(code)
        if ret != RET_OK:
            return ret, msg

        ret, msg = self._check_order_status(status_filter_list)
        if ret != RET_OK:
            return ret, msg

        if start:
            ret, data = normalize_date_format(start)
            if ret != RET_OK:
                return ret, data
            start = data

        if end:
            ret, data = normalize_date_format(end)
            if ret != RET_OK:
                return ret, data
            end = data

        query_processor = self._get_sync_query_processor(
            OrderListQuery.pack_req, OrderListQuery.unpack_rsp)

        # the keys of kargs should be corresponding to the actual function arguments
        kargs = {
            'order_id': str(order_id),
            'status_filter_list': status_filter_list,
            'code': str(stock_code),
            'start': str(start) if start else "",
            'end': str(end) if end else "",
            'trd_mkt': self.__trd_mkt,
            'trd_env': trd_env,
            'acc_id': acc_id,
            'conn_id': self.get_sync_conn_id(),
            'refresh_cache': refresh_cache
        }
        ret_code, msg, order_list = query_processor(**kargs)

        if ret_code != RET_OK:
            return RET_ERROR, msg

        return RET_OK, order_list

    def place_order(self, price, qty, code, trd_side, order_type=OrderType.NORMAL,
                    adjust_limit=0, trd_env=TrdEnv.REAL, acc_id=0, acc_index=0, remark=None,
                    time_in_force=TimeInForce.DAY, fill_outside_rth=False):
        """
        place order
        use  set_handle(HKTradeOrderHandlerBase) to recv order push !
        """
        # ret, msg = self._check_trd_env(trd_env)
        # if ret != RET_OK:
        #     return ret, msg
        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg

        ret, msg, acc_id = self._check_acc_id_and_acc_index(trd_env, acc_id, acc_index)
        if ret != RET_OK:
            return ret, msg

        ret, content = self._split_stock_code(code)
        if ret != RET_OK:
            return ret, content

        if remark is not None:
            if is_str(remark):
                remark_utf8 = remark.encode('utf-8')
                if len(remark_utf8) > 64:
                    return RET_ERROR, make_wrong_value_msg_utf8_len_le('remark', 64)
            else:
                return RET_ERROR, make_wrong_type_msg('remark', 'str')

        fill_outside_rth = True if fill_outside_rth else False

        market_str, stock_code = content

        query_processor = self._get_sync_query_processor(
            PlaceOrder.pack_req, PlaceOrder.unpack_rsp)

        # the keys of kargs should be corresponding to the actual function arguments
        kargs = {
            'trd_side': trd_side,
            'order_type': order_type,
            'price': float(price),
            'qty': float(qty),
            'code': stock_code,
            'adjust_limit': float(adjust_limit),
            'trd_mkt': self.__trd_mkt,
            'sec_mkt_str': market_str,
            'trd_env': trd_env,
            'acc_id': acc_id,
            'conn_id': self.get_sync_conn_id(),
            'remark': remark,
            'time_in_force': time_in_force,
            'fill_outside_rth': fill_outside_rth
        }

        ret_code, msg, order_id = query_processor(**kargs)
        if ret_code != RET_OK:
            return RET_ERROR, msg

        order_item = {'trd_env': trd_env, 'order_id': order_id}

        # 保持跟v2.0兼容， 增加必要的订单字段
        for x in range(3):
            ret_code, ret_data = self._order_list_query_impl(order_id=order_id,status_filter_list=[],
                                            code="", start="", end="", trd_env=trd_env, acc_id=acc_id,
                                                             refresh_cache=False)
            if ret_code == RET_OK and len(ret_data) > 0:
                order_item = ret_data[0]
                order_item['trd_env'] = trd_env
                break

        col_list = [
            "code", "stock_name", "trd_side", "order_type", "order_status",
            "order_id", "qty", "price", "create_time", "updated_time",
            "dealt_qty", "dealt_avg_price", "last_err_msg", "remark",
            "time_in_force", "fill_outside_rth"
        ]
        order_list = [order_item]
        order_table = pd.DataFrame(order_list, columns=col_list)

        return RET_OK, order_table

    def modify_order(self, modify_order_op, order_id, qty, price,
                     adjust_limit=0, trd_env=TrdEnv.REAL, acc_id=0, acc_index=0):

        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg

        ret, msg, acc_id = self._check_acc_id_and_acc_index(trd_env, acc_id, acc_index)
        if ret != RET_OK:
            return ret, msg

        if not order_id:
            return RET_ERROR, ERROR_STR_PREFIX + "the type of order_id param is wrong "

        if not ModifyOrderOp.if_has_key(modify_order_op):
            return RET_ERROR, ERROR_STR_PREFIX + "the type of modify_order_op param is wrong "

        query_processor = self._get_sync_query_processor(
            ModifyOrder.pack_req, ModifyOrder.unpack_rsp)

        kargs = {
            'modify_order_op': modify_order_op,
            'order_id': str(order_id),
            'price': float(price),
            'qty': float(qty),
            'adjust_limit': adjust_limit,
            'trd_mkt': self.__trd_mkt,
            'trd_env': trd_env,
            'acc_id': acc_id,
            'conn_id': self.get_sync_conn_id(),
        }

        ret_code, msg, modify_order_list = query_processor(**kargs)

        if ret_code != RET_OK:
            return RET_ERROR,msg

        col_list = ['trd_env', 'order_id']
        modify_order_table = pd.DataFrame(modify_order_list, columns=col_list)

        return RET_OK, modify_order_table

    def cancel_all_order(self, trd_env=TrdEnv.REAL, acc_id=0, acc_index=0):
        """
        取消所有的订单
        """
        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg

        ret, msg, acc_id = self._check_acc_id_and_acc_index(trd_env, acc_id, acc_index)
        if ret != RET_OK:
            return ret, msg

        query_processor = self._get_sync_query_processor(
            CancelOrder.pack_req, CancelOrder.unpack_rsp)

        kargs = {
            'trd_mkt': self.__trd_mkt,
            'trd_env': trd_env,
            'acc_id': acc_id,
            'conn_id': self.get_sync_conn_id(),
        }

        ret_code, msg, _ = query_processor(**kargs)
        return ret_code, msg


    def change_order(self, order_id, price, qty, adjust_limit=0, trd_env=TrdEnv.REAL, acc_id=0):
        return self.modify_order(ModifyOrderOp.NORMAL, order_id=order_id, qty=qty, price=price,
                                 adjust_limit=adjust_limit, trd_env=trd_env, acc_id=acc_id)

    def deal_list_query(self, code="", trd_env=TrdEnv.REAL, acc_id=0, acc_index=0, refresh_cache=False):
        """for querying deal list"""
        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg

        ret, msg, acc_id = self._check_acc_id_and_acc_index(trd_env, acc_id, acc_index)
        if ret != RET_OK:
            return ret, msg

        ret, msg, stock_code = self._check_stock_code(code)
        if ret != RET_OK:
            return ret, msg

        query_processor = self._get_sync_query_processor(
            DealListQuery.pack_req, DealListQuery.unpack_rsp)

        kargs = {
            'code': stock_code,
            'trd_mkt': self.__trd_mkt,
            'trd_env': trd_env,
            'acc_id': acc_id,
            'conn_id': self.get_sync_conn_id(),
            'refresh_cache': refresh_cache
            }
        ret_code, msg, deal_list = query_processor(**kargs)
        if ret_code != RET_OK:
            return RET_ERROR, msg

        col_list = [
            "code", "stock_name", "deal_id", "order_id", "qty", "price",
            "trd_side", "create_time", "counter_broker_id", "counter_broker_name", 'status'
        ]
        deal_list_table = pd.DataFrame(deal_list, columns=col_list)

        return RET_OK, deal_list_table

    def history_order_list_query(self, status_filter_list=[], code='', start='', end='',
                                 trd_env=TrdEnv.REAL, acc_id=0, acc_index=0):

        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg

        ret, msg, acc_id = self._check_acc_id_and_acc_index(trd_env, acc_id, acc_index)
        if ret != RET_OK:
            return ret, msg

        ret, msg, stock_code = self._check_stock_code(code)
        if ret != RET_OK:
            return ret, msg

        ret, msg = self._check_order_status(status_filter_list)
        if ret != RET_OK:
            return ret, msg

        ret, msg, start, end = normalize_start_end_date(start, end, 90)
        if ret != RET_OK:
            return ret, msg

        query_processor = self._get_sync_query_processor(
            HistoryOrderListQuery.pack_req,
            HistoryOrderListQuery.unpack_rsp)

        kargs = {
            'status_filter_list': status_filter_list,
            'code': str(stock_code),
            'start': str(start) if start else "",
            'end': str(end) if end else "",
            'trd_mkt': self.__trd_mkt,
            'trd_env': trd_env,
            'acc_id': acc_id,
            'conn_id': self.get_sync_conn_id()
        }
        ret_code, msg, order_list = query_processor(**kargs)
        if ret_code != RET_OK:
            return RET_ERROR, msg

        col_list = [
            "code", "stock_name", "trd_side", "order_type", "order_status",
            "order_id", "qty", "price", "create_time", "updated_time",
            "dealt_qty", "dealt_avg_price", "last_err_msg", "remark",
            "time_in_force", "fill_outside_rth"
        ]
        order_list_table = pd.DataFrame(order_list, columns=col_list)

        return RET_OK, order_list_table

    def history_deal_list_query(self, code='', start='', end='', trd_env=TrdEnv.REAL, acc_id=0, acc_index=0):

        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg

        ret, msg, acc_id = self._check_acc_id_and_acc_index(trd_env, acc_id, acc_index)
        if ret != RET_OK:
            return ret, msg

        ret, msg, stock_code = self._check_stock_code(code)
        if ret != RET_OK:
            return ret, msg

        ret, msg, start, end = normalize_start_end_date(start, end, 90)
        if ret != RET_OK:
            return ret, msg

        query_processor = self._get_sync_query_processor(
            HistoryDealListQuery.pack_req,
            HistoryDealListQuery.unpack_rsp)

        kargs = {
            'code': str(stock_code),
            'start': str(start) if start else "",
            'end': str(end) if end else "",
            'trd_mkt': self.__trd_mkt,
            'trd_env': trd_env,
            'acc_id': acc_id,
            'conn_id': self.get_sync_conn_id()
        }
        ret_code, msg, deal_list = query_processor(**kargs)
        if ret_code != RET_OK:
            return RET_ERROR, msg

        col_list = [
            "code", "stock_name", "deal_id", "order_id", "qty", "price",
            "trd_side", "create_time", "counter_broker_id", "counter_broker_name", 'status'
        ]
        deal_list_table = pd.DataFrame(deal_list, columns=col_list)

        return RET_OK, deal_list_table

    def acctradinginfo_query(self, order_type, code, price, order_id=None, adjust_limit=0, trd_env=TrdEnv.REAL, acc_id=0, acc_index=0):
        """
        查询账户下最大可买卖数量
        :param order_type: 订单类型，参见OrderType
        :param code: 证券代码，例如'HK.00700'
        :param price: 报价，3位精度
        :param order_id: 订单号。如果是新下单，则可以传None。如果是改单则要传单号。
        :param adjust_limit: 调整方向和调整幅度百分比限制，正数代表向上调整，负数代表向下调整，具体值代表调整幅度限制，如：0.015代表向上调整且幅度不超过1.5%；-0.01代表向下调整且幅度不超过1%。默认0表示不调整
        :param trd_env: 交易环境，参见TrdEnv
        :param acc_id: 业务账号，默认0表示第1个
        :param acc_index: int，交易业务子账户ID列表所对应的下标，默认0，表示第1个业务ID
        :return: (ret, data)

                ret == RET_OK, data为pd.DataFrame，数据列如下

                ret != RET_OK, data为错误信息

                =======================   ===========   ======================================================================================
                参数                       类型                        说明
                =======================   ===========   ======================================================================================
                max_cash_buy               float            不使用融资，仅自己的现金最大可买整手股数
                max_cash_and_margin_buy    float            使用融资，自己的现金 + 融资资金总共的最大可买整手股数
                max_position_sell          float            不使用融券(卖空)，仅自己的持仓最大可卖整手股数
                max_sell_short             float            使用融券(卖空)，最大可卖空整手股数，不包括多仓
                max_buy_back               float            卖空后，需要买回的最大整手股数。因为卖空后，必须先买回已卖空的股数，还掉股票，才能再继续买多。
                =======================   ===========   ======================================================================================
        """
        ret, msg = self._check_trd_env(trd_env)
        if ret != RET_OK:
            return ret, msg

        ret, msg, acc_id = self._check_acc_id_and_acc_index(trd_env, acc_id, acc_index)
        if ret != RET_OK:
            return ret, msg

        ret, content = self._split_stock_code(code)
        if ret != RET_OK:
            return ret, content

        market_str, stock_code = content

        query_processor = self._get_sync_query_processor(
            AccTradingInfoQuery.pack_req,
            AccTradingInfoQuery.unpack_rsp)

        kargs = {
            'order_type': order_type,
            'code': str(stock_code),
            'price': price,
            'order_id': order_id,
            'adjust_limit': adjust_limit,
            'trd_mkt': self.__trd_mkt,
            'sec_mkt_str': market_str,
            'trd_env': trd_env,
            'acc_id': acc_id,
            'conn_id': self.get_sync_conn_id()
        }

        ret_code, msg, data = query_processor(**kargs)
        if ret_code != RET_OK:
            return RET_ERROR, msg

        col_list = ['max_cash_buy', 'max_cash_and_margin_buy', 'max_position_sell', 'max_sell_short', 'max_buy_back']
        acctradinginfo_table = pd.DataFrame(data, columns=col_list)
        return RET_OK, acctradinginfo_table


# 港股交易接口
class OpenHKTradeContext(OpenTradeContextBase):
    def __init__(self, host="127.0.0.1", port=11111, is_encrypt=None, security_firm=SecurityFirm.FUTUSECURITIES):
        super(OpenHKTradeContext, self).__init__(TrdMarket.HK, host, port, is_encrypt=is_encrypt, security_firm=security_firm)


# 美股交易接口
class OpenUSTradeContext(OpenTradeContextBase):
    def __init__(self, host="127.0.0.1", port=11111, is_encrypt=None, security_firm=SecurityFirm.FUTUSECURITIES):
        super(OpenUSTradeContext, self).__init__(TrdMarket.US, host, port, is_encrypt=is_encrypt, security_firm=security_firm)


# A股通交易接口
class OpenHKCCTradeContext(OpenTradeContextBase):
    def __init__(self, host="127.0.0.1", port=11111, is_encrypt=None, security_firm=SecurityFirm.FUTUSECURITIES):
        super().__init__(TrdMarket.HKCC, host, port, is_encrypt=is_encrypt, security_firm=security_firm)

    def change_orde(self, *args, **kwargs):
        """不支持此接口"""
        return RET_ERROR, 'API not supported'


# A股交易接口
class OpenCNTradeContext(OpenTradeContextBase):
    def __init__(self, host="127.0.0.1", port=11111, is_encrypt=None, security_firm=SecurityFirm.FUTUSECURITIES):
        super(OpenCNTradeContext, self).__init__(TrdMarket.CN, host, port, is_encrypt=is_encrypt, security_firm=security_firm)


# 期货交易接口
class OpenFutureTradeContext(OpenTradeContextBase):
    def __init__(self, host="127.0.0.1", port=11111, is_encrypt=None, security_firm=SecurityFirm.FUTUSECURITIES):
        super(OpenFutureTradeContext, self).__init__(TrdMarket.FUTURES, host, port, is_encrypt=is_encrypt, security_firm=security_firm)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Qot_GetCodeChange.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Common_pb2 as Common__pb2
import Qot_Common_pb2 as Qot__Common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Qot_GetCodeChange.proto',
  package='Qot_GetCodeChange',
  syntax='proto2',
  serialized_pb=_b('\n\x17Qot_GetCodeChange.proto\x12\x11Qot_GetCodeChange\x1a\x0c\x43ommon.proto\x1a\x10Qot_Common.proto\"\xfc\x01\n\x0e\x43odeChangeInfo\x12\x0c\n\x04type\x18\x01 \x02(\x05\x12&\n\x08security\x18\x02 \x02(\x0b\x32\x14.Qot_Common.Security\x12-\n\x0frelatedSecurity\x18\x03 \x02(\x0b\x32\x14.Qot_Common.Security\x12\x12\n\npublicTime\x18\x04 \x01(\t\x12\x17\n\x0fpublicTimestamp\x18\x05 \x01(\x01\x12\x15\n\reffectiveTime\x18\x06 \x01(\t\x12\x1a\n\x12\x65\x66\x66\x65\x63tiveTimestamp\x18\x07 \x01(\x01\x12\x0f\n\x07\x65ndTime\x18\x08 \x01(\t\x12\x14\n\x0c\x65ndTimestamp\x18\t \x01(\x01\">\n\nTimeFilter\x12\x0c\n\x04type\x18\x01 \x02(\x05\x12\x11\n\tbeginTime\x18\x02 \x01(\t\x12\x0f\n\x07\x65ndTime\x18\x03 \x01(\t\"\x8f\x01\n\x03\x43\x32S\x12\x13\n\x0bplaceHolder\x18\x01 \x01(\x05\x12*\n\x0csecurityList\x18\x02 \x03(\x0b\x32\x14.Qot_Common.Security\x12\x35\n\x0etimeFilterList\x18\x03 \x03(\x0b\x32\x1d.Qot_GetCodeChange.TimeFilter\x12\x10\n\x08typeList\x18\x04 \x03(\x05\"@\n\x03S2C\x12\x39\n\x0e\x63odeChangeList\x18\x01 \x03(\x0b\x32!.Qot_GetCodeChange.CodeChangeInfo\".\n\x07Request\x12#\n\x03\x63\x32s\x18\x01 \x02(\x0b\x32\x16.Qot_GetCodeChange.C2S\"g\n\x08Response\x12\x15\n\x07retType\x18\x01 \x02(\x05:\x04-400\x12\x0e\n\x06retMsg\x18\x02 \x01(\t\x12\x0f\n\x07\x65rrCode\x18\x03 \x01(\x05\x12#\n\x03s2c\x18\x04 \x01(\x0b\x32\x16.Qot_GetCodeChange.S2C*\x8e\x02\n\x0e\x43odeChangeType\x12\x19\n\x15\x43odeChangeType_Unkown\x10\x00\x12\x1c\n\x18\x43odeChangeType_GemToMain\x10\x01\x12\x19\n\x15\x43odeChangeType_Unpaid\x10\x02\x12\x1c\n\x18\x43odeChangeType_ChangeLot\x10\x03\x12\x18\n\x14\x43odeChangeType_Split\x10\x04\x12\x18\n\x14\x43odeChangeType_Joint\x10\x05\x12\x1d\n\x19\x43odeChangeType_JointSplit\x10\x06\x12\x1d\n\x19\x43odeChangeType_SplitJoint\x10\x07\x12\x18\n\x14\x43odeChangeType_Other\x10\x08*|\n\x0eTimeFilterType\x12\x19\n\x15TimeFilterType_Unknow\x10\x00\x12\x19\n\x15TimeFilterType_Public\x10\x01\x12\x1c\n\x18TimeFilterType_Effective\x10\x02\x12\x16\n\x12TimeFilterType_End\x10\x03\x42G\n\x13\x63om.futu.openapi.pbZ0github.com/futuopen/ftapi4go/pb/qotgetcodechange')
  ,
  dependencies=[Common__pb2.DESCRIPTOR,Qot__Common__pb2.DESCRIPTOR,])

_CODECHANGETYPE = _descriptor.EnumDescriptor(
  name='CodeChangeType',
  full_name='Qot_GetCodeChange.CodeChangeType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CodeChangeType_Unkown', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CodeChangeType_GemToMain', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CodeChangeType_Unpaid', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CodeChangeType_ChangeLot', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CodeChangeType_Split', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CodeChangeType_Joint', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CodeChangeType_JointSplit', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CodeChangeType_SplitJoint', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CodeChangeType_Other', index=8, number=8,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=763,
  serialized_end=1033,
)
_sym_db.RegisterEnumDescriptor(_CODECHANGETYPE)

CodeChangeType = enum_type_wrapper.EnumTypeWrapper(_CODECHANGETYPE)
_TIMEFILTERTYPE = _descriptor.EnumDescriptor(
  name='TimeFilterType',
  full_name='Qot_GetCodeChange.TimeFilterType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TimeFilterType_Unknow', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TimeFilterType_Public', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TimeFilterType_Effective', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TimeFilterType_End', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1035,
  serialized_end=1159,
)
_sym_db.RegisterEnumDescriptor(_TIMEFILTERTYPE)

TimeFilterType = enum_type_wrapper.EnumTypeWrapper(_TIMEFILTERTYPE)
CodeChangeType_Unkown = 0
CodeChangeType_GemToMain = 1
CodeChangeType_Unpaid = 2
CodeChangeType_ChangeLot = 3
CodeChangeType_Split = 4
CodeChangeType_Joint = 5
CodeChangeType_JointSplit = 6
CodeChangeType_SplitJoint = 7
CodeChangeType_Other = 8
TimeFilterType_Unknow = 0
TimeFilterType_Public = 1
TimeFilterType_Effective = 2
TimeFilterType_End = 3



_CODECHANGEINFO = _descriptor.Descriptor(
  name='CodeChangeInfo',
  full_name='Qot_GetCodeChange.CodeChangeInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='Qot_GetCodeChange.CodeChangeInfo.type', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='security', full_name='Qot_GetCodeChange.CodeChangeInfo.security', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relatedSecurity', full_name='Qot_GetCodeChange.CodeChangeInfo.relatedSecurity', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='publicTime', full_name='Qot_GetCodeChange.CodeChangeInfo.publicTime', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='publicTimestamp', full_name='Qot_GetCodeChange.CodeChangeInfo.publicTimestamp', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='effectiveTime', full_name='Qot_GetCodeChange.CodeChangeInfo.effectiveTime', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='effectiveTimestamp', full_name='Qot_GetCodeChange.CodeChangeInfo.effectiveTimestamp', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endTime', full_name='Qot_GetCodeChange.CodeChangeInfo.endTime', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endTimestamp', full_name='Qot_GetCodeChange.CodeChangeInfo.endTimestamp', index=8,
      number=9, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=79,
  serialized_end=331,
)


_TIMEFILTER = _descriptor.Descriptor(
  name='TimeFilter',
  full_name='Qot_GetCodeChange.TimeFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='Qot_GetCodeChange.TimeFilter.type', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='beginTime', full_name='Qot_GetCodeChange.TimeFilter.beginTime', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endTime', full_name='Qot_GetCodeChange.TimeFilter.endTime', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=333,
  serialized_end=395,
)


_C2S = _descriptor.Descriptor(
  name='C2S',
  full_name='Qot_GetCodeChange.C2S',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='placeHolder', full_name='Qot_GetCodeChange.C2S.placeHolder', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='securityList', full_name='Qot_GetCodeChange.C2S.securityList', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeFilterList', full_name='Qot_GetCodeChange.C2S.timeFilterList', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='typeList', full_name='Qot_GetCodeChange.C2S.typeList', index=3,
      number=4, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=398,
  serialized_end=541,
)


_S2C = _descriptor.Descriptor(
  name='S2C',
  full_name='Qot_GetCodeChange.S2C',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='codeChangeList', full_name='Qot_GetCodeChange.S2C.codeChangeList', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=543,
  serialized_end=607,
)


_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='Qot_GetCodeChange.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='c2s', full_name='Qot_GetCodeChange.Request.c2s', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=609,
  serialized_end=655,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='Qot_GetCodeChange.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='retType', full_name='Qot_GetCodeChange.Response.retType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-400,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='retMsg', full_name='Qot_GetCodeChange.Response.retMsg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errCode', full_name='Qot_GetCodeChange.Response.errCode', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='s2c', full_name='Qot_GetCodeChange.Response.s2c', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=657,
  serialized_end=760,
)

_CODECHANGEINFO.fields_by_name['security'].message_type = Qot__Common__pb2._SECURITY
_CODECHANGEINFO.fields_by_name['relatedSecurity'].message_type = Qot__Common__pb2._SECURITY
_C2S.fields_by_name['securityList'].message_type = Qot__Common__pb2._SECURITY
_C2S.fields_by_name['timeFilterList'].message_type = _TIMEFILTER
_S2C.fields_by_name['codeChangeList'].message_type = _CODECHANGEINFO
_REQUEST.fields_by_name['c2s'].message_type = _C2S
_RESPONSE.fields_by_name['s2c'].message_type = _S2C
DESCRIPTOR.message_types_by_name['CodeChangeInfo'] = _CODECHANGEINFO
DESCRIPTOR.message_types_by_name['TimeFilter'] = _TIMEFILTER
DESCRIPTOR.message_types_by_name['C2S'] = _C2S
DESCRIPTOR.message_types_by_name['S2C'] = _S2C
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.enum_types_by_name['CodeChangeType'] = _CODECHANGETYPE
DESCRIPTOR.enum_types_by_name['TimeFilterType'] = _TIMEFILTERTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CodeChangeInfo = _reflection.GeneratedProtocolMessageType('CodeChangeInfo', (_message.Message,), dict(
  DESCRIPTOR = _CODECHANGEINFO,
  __module__ = 'Qot_GetCodeChange_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetCodeChange.CodeChangeInfo)
  ))
_sym_db.RegisterMessage(CodeChangeInfo)

TimeFilter = _reflection.GeneratedProtocolMessageType('TimeFilter', (_message.Message,), dict(
  DESCRIPTOR = _TIMEFILTER,
  __module__ = 'Qot_GetCodeChange_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetCodeChange.TimeFilter)
  ))
_sym_db.RegisterMessage(TimeFilter)

C2S = _reflection.GeneratedProtocolMessageType('C2S', (_message.Message,), dict(
  DESCRIPTOR = _C2S,
  __module__ = 'Qot_GetCodeChange_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetCodeChange.C2S)
  ))
_sym_db.RegisterMessage(C2S)

S2C = _reflection.GeneratedProtocolMessageType('S2C', (_message.Message,), dict(
  DESCRIPTOR = _S2C,
  __module__ = 'Qot_GetCodeChange_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetCodeChange.S2C)
  ))
_sym_db.RegisterMessage(S2C)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'Qot_GetCodeChange_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetCodeChange.Request)
  ))
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'Qot_GetCodeChange_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetCodeChange.Response)
  ))
_sym_db.RegisterMessage(Response)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023com.futu.openapi.pbZ0github.com/futuopen/ftapi4go/pb/qotgetcodechange'))
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Notify.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Common_pb2 as Common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Notify.proto',
  package='Notify',
  syntax='proto2',
  serialized_pb=_b('\n\x0cNotify.proto\x12\x06Notify\x1a\x0c\x43ommon.proto\"+\n\x08GtwEvent\x12\x11\n\teventType\x18\x01 \x02(\x05\x12\x0c\n\x04\x64\x65sc\x18\x02 \x02(\t\"=\n\rProgramStatus\x12,\n\rprogramStatus\x18\x01 \x02(\x0b\x32\x15.Common.ProgramStatus\"7\n\rConnectStatus\x12\x12\n\nqotLogined\x18\x01 \x02(\x08\x12\x12\n\ntrdLogined\x18\x02 \x02(\x08\"\xcb\x01\n\x08QotRight\x12\x12\n\nhkQotRight\x18\x04 \x02(\x05\x12\x12\n\nusQotRight\x18\x05 \x02(\x05\x12\x12\n\ncnQotRight\x18\x06 \x02(\x05\x12\x18\n\x10hkOptionQotRight\x18\x07 \x01(\x05\x12\x1b\n\x13hasUSOptionQotRight\x18\x08 \x01(\x08\x12\x18\n\x10hkFutureQotRight\x18\t \x01(\x05\x12\x18\n\x10usFutureQotRight\x18\n \x01(\x05\x12\x18\n\x10usOptionQotRight\x18\x0b \x01(\x05\"\x1c\n\x08\x41PILevel\x12\x10\n\x08\x61piLevel\x18\x03 \x02(\t\"4\n\x08\x41PIQuota\x12\x10\n\x08subQuota\x18\x01 \x02(\x05\x12\x16\n\x0ehistoryKLQuota\x18\x02 \x02(\x05\"\xfc\x01\n\x03S2C\x12\x0c\n\x04type\x18\x01 \x02(\x05\x12\x1f\n\x05\x65vent\x18\x02 \x01(\x0b\x32\x10.Notify.GtwEvent\x12,\n\rprogramStatus\x18\x03 \x01(\x0b\x32\x15.Notify.ProgramStatus\x12,\n\rconnectStatus\x18\x04 \x01(\x0b\x32\x15.Notify.ConnectStatus\x12\"\n\x08qotRight\x18\x05 \x01(\x0b\x32\x10.Notify.QotRight\x12\"\n\x08\x61piLevel\x18\x06 \x01(\x0b\x32\x10.Notify.APILevel\x12\"\n\x08\x61piQuota\x18\x07 \x01(\x0b\x32\x10.Notify.APIQuota\"\\\n\x08Response\x12\x15\n\x07retType\x18\x01 \x02(\x05:\x04-400\x12\x0e\n\x06retMsg\x18\x02 \x01(\t\x12\x0f\n\x07\x65rrCode\x18\x03 \x01(\x05\x12\x18\n\x03s2c\x18\x04 \x01(\x0b\x32\x0b.Notify.S2C*\xbe\x01\n\nNotifyType\x12\x13\n\x0fNotifyType_None\x10\x00\x12\x17\n\x13NotifyType_GtwEvent\x10\x01\x12\x1c\n\x18NotifyType_ProgramStatus\x10\x02\x12\x19\n\x15NotifyType_ConnStatus\x10\x03\x12\x17\n\x13NotifyType_QotRight\x10\x04\x12\x17\n\x13NotifyType_APILevel\x10\x05\x12\x17\n\x13NotifyType_APIQuota\x10\x06*\x9b\x04\n\x0cGtwEventType\x12\x15\n\x11GtwEventType_None\x10\x00\x12#\n\x1fGtwEventType_LocalCfgLoadFailed\x10\x01\x12 \n\x1cGtwEventType_APISvrRunFailed\x10\x02\x12\x1c\n\x18GtwEventType_ForceUpdate\x10\x03\x12\x1c\n\x18GtwEventType_LoginFailed\x10\x04\x12\"\n\x1eGtwEventType_UnAgreeDisclaimer\x10\x05\x12\x1e\n\x1aGtwEventType_NetCfgMissing\x10\x06\x12\x1a\n\x16GtwEventType_KickedOut\x10\x07\x12 \n\x1cGtwEventType_LoginPwdChanged\x10\x08\x12\x19\n\x15GtwEventType_BanLogin\x10\t\x12\"\n\x1eGtwEventType_NeedPicVerifyCode\x10\n\x12$\n GtwEventType_NeedPhoneVerifyCode\x10\x0b\x12 \n\x1cGtwEventType_AppDataNotExist\x10\x0c\x12#\n\x1fGtwEventType_NessaryDataMissing\x10\r\x12 \n\x1cGtwEventType_TradePwdChanged\x10\x0e\x12!\n\x1dGtwEventType_EnableDeviceLock\x10\x0f\x42=\n\x13\x63om.futu.openapi.pbZ&github.com/futuopen/ftapi4go/pb/notify')
  ,
  dependencies=[Common__pb2.DESCRIPTOR,])

_NOTIFYTYPE = _descriptor.EnumDescriptor(
  name='NotifyType',
  full_name='Notify.NotifyType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NotifyType_None', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NotifyType_GtwEvent', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NotifyType_ProgramStatus', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NotifyType_ConnStatus', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NotifyType_QotRight', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NotifyType_APILevel', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NotifyType_APIQuota', index=6, number=6,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=843,
  serialized_end=1033,
)
_sym_db.RegisterEnumDescriptor(_NOTIFYTYPE)

NotifyType = enum_type_wrapper.EnumTypeWrapper(_NOTIFYTYPE)
_GTWEVENTTYPE = _descriptor.EnumDescriptor(
  name='GtwEventType',
  full_name='Notify.GtwEventType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_None', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_LocalCfgLoadFailed', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_APISvrRunFailed', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_ForceUpdate', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_LoginFailed', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_UnAgreeDisclaimer', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_NetCfgMissing', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_KickedOut', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_LoginPwdChanged', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_BanLogin', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_NeedPicVerifyCode', index=10, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_NeedPhoneVerifyCode', index=11, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_AppDataNotExist', index=12, number=12,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_NessaryDataMissing', index=13, number=13,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_TradePwdChanged', index=14, number=14,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GtwEventType_EnableDeviceLock', index=15, number=15,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1036,
  serialized_end=1575,
)
_sym_db.RegisterEnumDescriptor(_GTWEVENTTYPE)

GtwEventType = enum_type_wrapper.EnumTypeWrapper(_GTWEVENTTYPE)
NotifyType_None = 0
NotifyType_GtwEvent = 1
NotifyType_ProgramStatus = 2
NotifyType_ConnStatus = 3
NotifyType_QotRight = 4
NotifyType_APILevel = 5
NotifyType_APIQuota = 6
GtwEventType_None = 0
GtwEventType_LocalCfgLoadFailed = 1
GtwEventType_APISvrRunFailed = 2
GtwEventType_ForceUpdate = 3
GtwEventType_LoginFailed = 4
GtwEventType_UnAgreeDisclaimer = 5
GtwEventType_NetCfgMissing = 6
GtwEventType_KickedOut = 7
GtwEventType_LoginPwdChanged = 8
GtwEventType_BanLogin = 9
GtwEventType_NeedPicVerifyCode = 10
GtwEventType_NeedPhoneVerifyCode = 11
GtwEventType_AppDataNotExist = 12
GtwEventType_NessaryDataMissing = 13
GtwEventType_TradePwdChanged = 14
GtwEventType_EnableDeviceLock = 15



_GTWEVENT = _descriptor.Descriptor(
  name='GtwEvent',
  full_name='Notify.GtwEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eventType', full_name='Notify.GtwEvent.eventType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='desc', full_name='Notify.GtwEvent.desc', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=38,
  serialized_end=81,
)


_PROGRAMSTATUS = _descriptor.Descriptor(
  name='ProgramStatus',
  full_name='Notify.ProgramStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='programStatus', full_name='Notify.ProgramStatus.programStatus', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=83,
  serialized_end=144,
)


_CONNECTSTATUS = _descriptor.Descriptor(
  name='ConnectStatus',
  full_name='Notify.ConnectStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='qotLogined', full_name='Notify.ConnectStatus.qotLogined', index=0,
      number=1, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trdLogined', full_name='Notify.ConnectStatus.trdLogined', index=1,
      number=2, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=146,
  serialized_end=201,
)


_QOTRIGHT = _descriptor.Descriptor(
  name='QotRight',
  full_name='Notify.QotRight',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hkQotRight', full_name='Notify.QotRight.hkQotRight', index=0,
      number=4, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='usQotRight', full_name='Notify.QotRight.usQotRight', index=1,
      number=5, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cnQotRight', full_name='Notify.QotRight.cnQotRight', index=2,
      number=6, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hkOptionQotRight', full_name='Notify.QotRight.hkOptionQotRight', index=3,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hasUSOptionQotRight', full_name='Notify.QotRight.hasUSOptionQotRight', index=4,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hkFutureQotRight', full_name='Notify.QotRight.hkFutureQotRight', index=5,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='usFutureQotRight', full_name='Notify.QotRight.usFutureQotRight', index=6,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='usOptionQotRight', full_name='Notify.QotRight.usOptionQotRight', index=7,
      number=11, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=204,
  serialized_end=407,
)


_APILEVEL = _descriptor.Descriptor(
  name='APILevel',
  full_name='Notify.APILevel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='apiLevel', full_name='Notify.APILevel.apiLevel', index=0,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=409,
  serialized_end=437,
)


_APIQUOTA = _descriptor.Descriptor(
  name='APIQuota',
  full_name='Notify.APIQuota',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='subQuota', full_name='Notify.APIQuota.subQuota', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='historyKLQuota', full_name='Notify.APIQuota.historyKLQuota', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=439,
  serialized_end=491,
)


_S2C = _descriptor.Descriptor(
  name='S2C',
  full_name='Notify.S2C',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='Notify.S2C.type', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='event', full_name='Notify.S2C.event', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='programStatus', full_name='Notify.S2C.programStatus', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='connectStatus', full_name='Notify.S2C.connectStatus', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='qotRight', full_name='Notify.S2C.qotRight', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='apiLevel', full_name='Notify.S2C.apiLevel', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='apiQuota', full_name='Notify.S2C.apiQuota', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=494,
  serialized_end=746,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='Notify.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='retType', full_name='Notify.Response.retType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-400,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='retMsg', full_name='Notify.Response.retMsg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errCode', full_name='Notify.Response.errCode', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='s2c', full_name='Notify.Response.s2c', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=748,
  serialized_end=840,
)

_PROGRAMSTATUS.fields_by_name['programStatus'].message_type = Common__pb2._PROGRAMSTATUS
_S2C.fields_by_name['event'].message_type = _GTWEVENT
_S2C.fields_by_name['programStatus'].message_type = _PROGRAMSTATUS
_S2C.fields_by_name['connectStatus'].message_type = _CONNECTSTATUS
_S2C.fields_by_name['qotRight'].message_type = _QOTRIGHT
_S2C.fields_by_name['apiLevel'].message_type = _APILEVEL
_S2C.fields_by_name['apiQuota'].message_type = _APIQUOTA
_RESPONSE.fields_by_name['s2c'].message_type = _S2C
DESCRIPTOR.message_types_by_name['GtwEvent'] = _GTWEVENT
DESCRIPTOR.message_types_by_name['ProgramStatus'] = _PROGRAMSTATUS
DESCRIPTOR.message_types_by_name['ConnectStatus'] = _CONNECTSTATUS
DESCRIPTOR.message_types_by_name['QotRight'] = _QOTRIGHT
DESCRIPTOR.message_types_by_name['APILevel'] = _APILEVEL
DESCRIPTOR.message_types_by_name['APIQuota'] = _APIQUOTA
DESCRIPTOR.message_types_by_name['S2C'] = _S2C
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.enum_types_by_name['NotifyType'] = _NOTIFYTYPE
DESCRIPTOR.enum_types_by_name['GtwEventType'] = _GTWEVENTTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GtwEvent = _reflection.GeneratedProtocolMessageType('GtwEvent', (_message.Message,), dict(
  DESCRIPTOR = _GTWEVENT,
  __module__ = 'Notify_pb2'
  # @@protoc_insertion_point(class_scope:Notify.GtwEvent)
  ))
_sym_db.RegisterMessage(GtwEvent)

ProgramStatus = _reflection.GeneratedProtocolMessageType('ProgramStatus', (_message.Message,), dict(
  DESCRIPTOR = _PROGRAMSTATUS,
  __module__ = 'Notify_pb2'
  # @@protoc_insertion_point(class_scope:Notify.ProgramStatus)
  ))
_sym_db.RegisterMessage(ProgramStatus)

ConnectStatus = _reflection.GeneratedProtocolMessageType('ConnectStatus', (_message.Message,), dict(
  DESCRIPTOR = _CONNECTSTATUS,
  __module__ = 'Notify_pb2'
  # @@protoc_insertion_point(class_scope:Notify.ConnectStatus)
  ))
_sym_db.RegisterMessage(ConnectStatus)

QotRight = _reflection.GeneratedProtocolMessageType('QotRight', (_message.Message,), dict(
  DESCRIPTOR = _QOTRIGHT,
  __module__ = 'Notify_pb2'
  # @@protoc_insertion_point(class_scope:Notify.QotRight)
  ))
_sym_db.RegisterMessage(QotRight)

APILevel = _reflection.GeneratedProtocolMessageType('APILevel', (_message.Message,), dict(
  DESCRIPTOR = _APILEVEL,
  __module__ = 'Notify_pb2'
  # @@protoc_insertion_point(class_scope:Notify.APILevel)
  ))
_sym_db.RegisterMessage(APILevel)

APIQuota = _reflection.GeneratedProtocolMessageType('APIQuota', (_message.Message,), dict(
  DESCRIPTOR = _APIQUOTA,
  __module__ = 'Notify_pb2'
  # @@protoc_insertion_point(class_scope:Notify.APIQuota)
  ))
_sym_db.RegisterMessage(APIQuota)

S2C = _reflection.GeneratedProtocolMessageType('S2C', (_message.Message,), dict(
  DESCRIPTOR = _S2C,
  __module__ = 'Notify_pb2'
  # @@protoc_insertion_point(class_scope:Notify.S2C)
  ))
_sym_db.RegisterMessage(S2C)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'Notify_pb2'
  # @@protoc_insertion_point(class_scope:Notify.Response)
  ))
_sym_db.RegisterMessage(Response)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023com.futu.openapi.pbZ&github.com/futuopen/ftapi4go/pb/notify'))
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Qot_GetHistoryKLPoints.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Common_pb2 as Common__pb2
import Qot_Common_pb2 as Qot__Common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Qot_GetHistoryKLPoints.proto',
  package='Qot_GetHistoryKLPoints',
  syntax='proto2',
  serialized_pb=_b('\n\x1cQot_GetHistoryKLPoints.proto\x12\x16Qot_GetHistoryKLPoints\x1a\x0c\x43ommon.proto\x1a\x10Qot_Common.proto\"\xaf\x01\n\x03\x43\x32S\x12\x11\n\trehabType\x18\x01 \x02(\x05\x12\x0e\n\x06klType\x18\x02 \x02(\x05\x12\x12\n\nnoDataMode\x18\x03 \x02(\x05\x12*\n\x0csecurityList\x18\x04 \x03(\x0b\x32\x14.Qot_Common.Security\x12\x10\n\x08timeList\x18\x05 \x03(\t\x12\x19\n\x11maxReqSecurityNum\x18\x06 \x01(\x05\x12\x18\n\x10needKLFieldsFlag\x18\x07 \x01(\x03\"Q\n\x0fHistoryPointsKL\x12\x0e\n\x06status\x18\x01 \x02(\x05\x12\x0f\n\x07reqTime\x18\x02 \x02(\t\x12\x1d\n\x02kl\x18\x03 \x02(\x0b\x32\x11.Qot_Common.KLine\"z\n\x17SecurityHistoryKLPoints\x12&\n\x08security\x18\x01 \x02(\x0b\x32\x14.Qot_Common.Security\x12\x37\n\x06klList\x18\x02 \x03(\x0b\x32\'.Qot_GetHistoryKLPoints.HistoryPointsKL\"\\\n\x03S2C\x12\x44\n\x0bklPointList\x18\x01 \x03(\x0b\x32/.Qot_GetHistoryKLPoints.SecurityHistoryKLPoints\x12\x0f\n\x07hasNext\x18\x02 \x01(\x08\"3\n\x07Request\x12(\n\x03\x63\x32s\x18\x01 \x02(\x0b\x32\x1b.Qot_GetHistoryKLPoints.C2S\"l\n\x08Response\x12\x15\n\x07retType\x18\x01 \x02(\x05:\x04-400\x12\x0e\n\x06retMsg\x18\x02 \x01(\t\x12\x0f\n\x07\x65rrCode\x18\x03 \x01(\x05\x12(\n\x03s2c\x18\x04 \x01(\x0b\x32\x1b.Qot_GetHistoryKLPoints.S2C*R\n\nNoDataMode\x12\x13\n\x0fNoDataMode_Null\x10\x00\x12\x16\n\x12NoDataMode_Forward\x10\x01\x12\x17\n\x13NoDataMode_Backward\x10\x02*g\n\nDataStatus\x12\x13\n\x0f\x44\x61taStatus_Null\x10\x00\x12\x16\n\x12\x44\x61taStatus_Current\x10\x01\x12\x17\n\x13\x44\x61taStatus_Previous\x10\x02\x12\x13\n\x0f\x44\x61taStatus_Back\x10\x03\x42L\n\x13\x63om.futu.openapi.pbZ5github.com/futuopen/ftapi4go/pb/qotgethistoryklpoints')
  ,
  dependencies=[Common__pb2.DESCRIPTOR,Qot__Common__pb2.DESCRIPTOR,])

_NODATAMODE = _descriptor.EnumDescriptor(
  name='NoDataMode',
  full_name='Qot_GetHistoryKLPoints.NoDataMode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NoDataMode_Null', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NoDataMode_Forward', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NoDataMode_Backward', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=730,
  serialized_end=812,
)
_sym_db.RegisterEnumDescriptor(_NODATAMODE)

NoDataMode = enum_type_wrapper.EnumTypeWrapper(_NODATAMODE)
_DATASTATUS = _descriptor.EnumDescriptor(
  name='DataStatus',
  full_name='Qot_GetHistoryKLPoints.DataStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DataStatus_Null', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DataStatus_Current', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DataStatus_Previous', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DataStatus_Back', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=814,
  serialized_end=917,
)
_sym_db.RegisterEnumDescriptor(_DATASTATUS)

DataStatus = enum_type_wrapper.EnumTypeWrapper(_DATASTATUS)
NoDataMode_Null = 0
NoDataMode_Forward = 1
NoDataMode_Backward = 2
DataStatus_Null = 0
DataStatus_Current = 1
DataStatus_Previous = 2
DataStatus_Back = 3



_C2S = _descriptor.Descriptor(
  name='C2S',
  full_name='Qot_GetHistoryKLPoints.C2S',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rehabType', full_name='Qot_GetHistoryKLPoints.C2S.rehabType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='klType', full_name='Qot_GetHistoryKLPoints.C2S.klType', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noDataMode', full_name='Qot_GetHistoryKLPoints.C2S.noDataMode', index=2,
      number=3, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='securityList', full_name='Qot_GetHistoryKLPoints.C2S.securityList', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeList', full_name='Qot_GetHistoryKLPoints.C2S.timeList', index=4,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxReqSecurityNum', full_name='Qot_GetHistoryKLPoints.C2S.maxReqSecurityNum', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='needKLFieldsFlag', full_name='Qot_GetHistoryKLPoints.C2S.needKLFieldsFlag', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=89,
  serialized_end=264,
)


_HISTORYPOINTSKL = _descriptor.Descriptor(
  name='HistoryPointsKL',
  full_name='Qot_GetHistoryKLPoints.HistoryPointsKL',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='Qot_GetHistoryKLPoints.HistoryPointsKL.status', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reqTime', full_name='Qot_GetHistoryKLPoints.HistoryPointsKL.reqTime', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='kl', full_name='Qot_GetHistoryKLPoints.HistoryPointsKL.kl', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=266,
  serialized_end=347,
)


_SECURITYHISTORYKLPOINTS = _descriptor.Descriptor(
  name='SecurityHistoryKLPoints',
  full_name='Qot_GetHistoryKLPoints.SecurityHistoryKLPoints',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='security', full_name='Qot_GetHistoryKLPoints.SecurityHistoryKLPoints.security', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='klList', full_name='Qot_GetHistoryKLPoints.SecurityHistoryKLPoints.klList', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=349,
  serialized_end=471,
)


_S2C = _descriptor.Descriptor(
  name='S2C',
  full_name='Qot_GetHistoryKLPoints.S2C',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='klPointList', full_name='Qot_GetHistoryKLPoints.S2C.klPointList', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hasNext', full_name='Qot_GetHistoryKLPoints.S2C.hasNext', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=473,
  serialized_end=565,
)


_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='Qot_GetHistoryKLPoints.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='c2s', full_name='Qot_GetHistoryKLPoints.Request.c2s', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=567,
  serialized_end=618,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='Qot_GetHistoryKLPoints.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='retType', full_name='Qot_GetHistoryKLPoints.Response.retType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-400,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='retMsg', full_name='Qot_GetHistoryKLPoints.Response.retMsg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errCode', full_name='Qot_GetHistoryKLPoints.Response.errCode', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='s2c', full_name='Qot_GetHistoryKLPoints.Response.s2c', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=620,
  serialized_end=728,
)

_C2S.fields_by_name['securityList'].message_type = Qot__Common__pb2._SECURITY
_HISTORYPOINTSKL.fields_by_name['kl'].message_type = Qot__Common__pb2._KLINE
_SECURITYHISTORYKLPOINTS.fields_by_name['security'].message_type = Qot__Common__pb2._SECURITY
_SECURITYHISTORYKLPOINTS.fields_by_name['klList'].message_type = _HISTORYPOINTSKL
_S2C.fields_by_name['klPointList'].message_type = _SECURITYHISTORYKLPOINTS
_REQUEST.fields_by_name['c2s'].message_type = _C2S
_RESPONSE.fields_by_name['s2c'].message_type = _S2C
DESCRIPTOR.message_types_by_name['C2S'] = _C2S
DESCRIPTOR.message_types_by_name['HistoryPointsKL'] = _HISTORYPOINTSKL
DESCRIPTOR.message_types_by_name['SecurityHistoryKLPoints'] = _SECURITYHISTORYKLPOINTS
DESCRIPTOR.message_types_by_name['S2C'] = _S2C
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.enum_types_by_name['NoDataMode'] = _NODATAMODE
DESCRIPTOR.enum_types_by_name['DataStatus'] = _DATASTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

C2S = _reflection.GeneratedProtocolMessageType('C2S', (_message.Message,), dict(
  DESCRIPTOR = _C2S,
  __module__ = 'Qot_GetHistoryKLPoints_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetHistoryKLPoints.C2S)
  ))
_sym_db.RegisterMessage(C2S)

HistoryPointsKL = _reflection.GeneratedProtocolMessageType('HistoryPointsKL', (_message.Message,), dict(
  DESCRIPTOR = _HISTORYPOINTSKL,
  __module__ = 'Qot_GetHistoryKLPoints_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetHistoryKLPoints.HistoryPointsKL)
  ))
_sym_db.RegisterMessage(HistoryPointsKL)

SecurityHistoryKLPoints = _reflection.GeneratedProtocolMessageType('SecurityHistoryKLPoints', (_message.Message,), dict(
  DESCRIPTOR = _SECURITYHISTORYKLPOINTS,
  __module__ = 'Qot_GetHistoryKLPoints_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetHistoryKLPoints.SecurityHistoryKLPoints)
  ))
_sym_db.RegisterMessage(SecurityHistoryKLPoints)

S2C = _reflection.GeneratedProtocolMessageType('S2C', (_message.Message,), dict(
  DESCRIPTOR = _S2C,
  __module__ = 'Qot_GetHistoryKLPoints_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetHistoryKLPoints.S2C)
  ))
_sym_db.RegisterMessage(S2C)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'Qot_GetHistoryKLPoints_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetHistoryKLPoints.Request)
  ))
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'Qot_GetHistoryKLPoints_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetHistoryKLPoints.Response)
  ))
_sym_db.RegisterMessage(Response)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023com.futu.openapi.pbZ5github.com/futuopen/ftapi4go/pb/qotgethistoryklpoints'))
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
"""
Examples for use the python functions: get push data
"""
from time import sleep
from futu import *



class StockQuoteTest(StockQuoteHandlerBase):
    """
    获得报价推送数据
    """
    def on_recv_rsp(self, rsp_pb):
        """数据响应回调函数"""
        ret_code, content = super(StockQuoteTest, self).on_recv_rsp(rsp_pb)
        if ret_code != RET_OK:
            logger.debug("StockQuoteTest: error, msg: %s" % content)
            return RET_ERROR, content
        print("* StockQuoteTest : %s" % content)
        return RET_OK, content


class CurKlineTest(CurKlineHandlerBase):
    """ kline push"""
    def on_recv_rsp(self, rsp_pb):
        """数据响应回调函数"""
        ret_code, content = super(CurKlineTest, self).on_recv_rsp(rsp_pb)
        if ret_code != RET_OK:
            print("* CurKlineTest: error, msg: %s" % content)
        return RET_OK, content


class RTDataTest(RTDataHandlerBase):
    """ 获取分时推送数据 """
    def on_recv_rsp(self, rsp_pb):
        """数据响应回调函数"""
        ret_code, content = super(RTDataTest, self).on_recv_rsp(rsp_pb)
        if ret_code != RET_OK:
            print("* RTDataTest: error, msg: %s" % content)
            return RET_ERROR, content
        print("* RTDataTest :%s \n" % content)
        return RET_OK, content


class TickerTest(TickerHandlerBase):
    """ 获取逐笔推送数据 """
    def on_recv_rsp(self, rsp_pb):
        """数据响应回调函数"""
        ret_code, content = super(TickerTest, self).on_recv_rsp(rsp_pb)
        if ret_code != RET_OK:
            print("* TickerTest: error, msg: %s" % content)
            return RET_ERROR, content
        print("* TickerTest\n", content)
        return RET_OK, content


class OrderBookTest(OrderBookHandlerBase):
    """ 获得摆盘推送数据 """
    def on_recv_rsp(self, rsp_pb):
        """数据响应回调函数"""
        ret_code, content = super(OrderBookTest, self).on_recv_rsp(rsp_pb)
        if ret_code != RET_OK:
            print("* OrderBookTest: error, msg: %s" % content)
            return RET_ERROR, content
        print("* OrderBookTest\n", content)
        return RET_OK, content


class BrokerTest(BrokerHandlerBase):
    """ 获取经纪队列推送数据 """
    def on_recv_rsp(self, rsp_pb):
        """数据响应回调函数"""
        ret_code, stock_code, contents = super(BrokerTest, self).on_recv_rsp(rsp_pb)
        if ret_code == RET_OK:
            bid_content = contents[0]
            ask_content = contents[1]
            print("* BrokerTest code \n", stock_code)
            print("* BrokerTest bid \n", bid_content)
            print("* BrokerTest ask \n", ask_content)
        return ret_code


class SysNotifyTest(SysNotifyHandlerBase):
    """sys notify"""
    def on_recv_rsp(self, rsp_pb):
        """receive response callback function"""
        ret_code, content = super(SysNotifyTest, self).on_recv_rsp(rsp_pb)

        if ret_code == RET_OK:
            main_type, sub_type, msg = content
            print("* SysNotify main_type='{}' sub_type='{}' msg='{}'\n".format(main_type, sub_type, msg))
        else:
            print("* SysNotify error:{}\n".format(content))
        return ret_code, content


class TradeOrderTest(TradeOrderHandlerBase):
    """ order update push"""
    def on_recv_rsp(self, rsp_pb):
        ret, content = super(TradeOrderTest, self).on_recv_rsp(rsp_pb)

        if ret == RET_OK:
            print("* TradeOrderTest content={}\n".format(content))

        return ret, content


class TradeDealTest(TradeDealHandlerBase):
    """ order update push"""
    def on_recv_rsp(self, rsp_pb):
        ret, content = super(TradeDealTest, self).on_recv_rsp(rsp_pb)

        if ret == RET_OK:
            print("TradeDealTest content={}".format(content))

        return ret, content


def quote_test():
    '''
    行情接口调用测试
    :return:
    '''
    quote_ctx = OpenQuoteContext(host='127.0.0.1', port=11111)

    # 设置异步回调接口
    quote_ctx.set_handler(StockQuoteTest())
    quote_ctx.set_handler(CurKlineTest())
    quote_ctx.set_handler(RTDataTest())
    quote_ctx.set_handler(TickerTest())
    quote_ctx.set_handler(OrderBookTest())
    quote_ctx.set_handler(BrokerTest())
    quote_ctx.set_handler(SysNotifyTest())
    # quote_ctx.start()

    # 获取推送数据
    big_sub_codes = ['HK.02318', 'HK.02828', 'HK.00939', 'HK.01093', 'HK.01299', 'HK.00175',
                     'HK.01299', 'HK.01833', 'HK.00005', 'HK.00883', 'HK.00388', 'HK.01398',
                     'HK.01114', 'HK.02800', 'HK.02018', 'HK.03988', 'HK.00386', 'HK.01211',
                     'HK.00700', 'HK.01177',  'HK.02601', 'HK.02628', 'HK.HSImain']
    subtype_list = [SubType.QUOTE, SubType.ORDER_BOOK, SubType.TICKER, SubType.K_DAY, SubType.RT_DATA, SubType.BROKER]

    sub_codes =  ['HK.00700', 'HK.HSImain']

    # print("* get_owner_plate : {}\n".format(quote_ctx.get_owner_plate(code_list)))
    # print("* get_referencestock_list : {}\n".format(quote_ctx.get_referencestock_list(
    #     code_list[0], SecurityReferenceType.WARRANT)))
    # # print("* get_holding_change_list : {}\n".format(quote_ctx.get_holding_change_list(
    # #     "US.AAPL", StockHolder.EXECUTIVE, "2018-01-01", None)))
    #
    # print("* request_history_kline : {}\n".format(quote_ctx.request_history_kline(
    #     code_list[0], "2018-01-01", None, KLType.K_1M, AuType.QFQ, [KL_FIELD.ALL], 50000)))

    # 测试大量数据定阅
    # if len(big_sub_codes):
    #     print("* subscribe : {}\n".format(quote_ctx.subscribe(big_sub_codes, subtype_list)))

    """
    if True:
        print("* subscribe : {}\n".format(quote_ctx.subscribe(code_list, subtype_list)))
        print("* query_subscription : {}\n".format(quote_ctx.query_subscription(True)))
        sleep(60.1)
        print("* unsubscribe : {}\n".format(quote_ctx.unsubscribe(code_list, subtype_list)))
        print("* query_subscription : {}\n".format(quote_ctx.query_subscription(True)))
        sleep(1)
    """
    print("* subscribe : {}\n".format(quote_ctx.subscribe(sub_codes, subtype_list)))

    # # """
    # print("* get_stock_basicinfo : {}\n".format(quote_ctx.get_stock_basicinfo(Market.HK, SecurityType.ETF)))
    # print("* get_cur_kline : {}\n".format(quote_ctx.get_cur_kline(code_list[0], 10, SubType.K_DAY, AuType.QFQ)))
    #
    # print("* get_rt_data : {}\n".format(quote_ctx.get_rt_data(code_list[0])))
    # print("* get_rt_ticker : {}\n".format(quote_ctx.get_rt_ticker(code_list[0], 10)))
    #
    # print("* get_broker_queue : {}\n".format(quote_ctx.get_broker_queue(code_list[0])))
    # print("* get_order_book : {}\n".format(quote_ctx.get_order_book(code_list[0])))
    # print("* request_history_kline : {}\n".format(quote_ctx.request_history_kline('HK.00700', start='2017-06-20', end='2017-06-22')))
    # # """
    #
    # # """
    # print("* get_multi_points_history_kline : {}\n".format(quote_ctx.get_multi_points_history_kline(code_list, ['2017-06-20', '2017-06-22', '2017-06-23'], KL_FIELD.ALL,
    #                                                KLType.K_DAY, AuType.QFQ)))
    # print("* get_autype_list : {}\n".format(quote_ctx.get_autype_list("HK.00700")))
    #
    # print("* get_trading_days : {}\n".format(quote_ctx.get_trading_days(Market.HK, '2018-11-01', '2018-11-20')))
    #
    # print("* get_market_snapshot : {}\n".format(quote_ctx.get_market_snapshot('HK.21901')))
    # print("* get_market_snapshot : {}\n".format(quote_ctx.get_market_snapshot(code_list)))
    #
    # print("* get_plate_list : {}\n".format(quote_ctx.get_plate_list(Market.HK, Plate.ALL)))
    # print("* get_plate_stock : {}\n".format(quote_ctx.get_plate_stock('HK.BK1001')))
    # """

    # """
    sleep(15)
    quote_ctx.close()
    # """


def trade_hkcc_test():
    """
    A股通交易测试
    :return:
    """
    trd_ctx = OpenHKCCTradeContext(host='127.0.0.1', port=11111)
    trd_ctx.set_handler(TradeOrderTest())
    trd_ctx.set_handler(TradeDealTest())
    trd_ctx.start()

    # 交易请求必须先解锁 !!!
    pwd_unlock = '979899'
    print("* unlock_trade : {}\n".format(trd_ctx.unlock_trade(pwd_unlock)))

    print("* accinfo_query : {}\n".format(trd_ctx.accinfo_query()))
    print("* position_list_query : {}\n".format(trd_ctx.position_list_query(pl_ratio_min=-50, pl_ratio_max=50)))
    print("* order_list_query : {}\n".format(trd_ctx.order_list_query(status_filter_list=[OrderStatus.DISABLED])))
    print("* get_acc_list : {}\n".format(trd_ctx.get_acc_list()))
    print("* order_list_query : {}\n".format(trd_ctx.order_list_query(status_filter_list=[OrderStatus.SUBMITTED])))

    ret_code, ret_data = trd_ctx.place_order(0.1, 100, "SZ.000979", TrdSide.BUY)
    print("* place_order : {}\n".format(ret_data))
    if ret_code == RET_OK:
        order_id = ret_data['order_id'][0]
        print("* modify_order : {}\n".format(trd_ctx.modify_order(ModifyOrderOp.CANCEL, order_id, 0, 0)))

    print("* deal_list_query : {}\n".format(trd_ctx.deal_list_query(code="000979")))
    print("* history_order_list_query : {}\n".format(trd_ctx.history_order_list_query(status_filter_list=[OrderStatus.FILLED_ALL, OrderStatus.FILLED_PART],
                                           code="512310", start="", end="2018-2-1")))

    print("* history_deal_list_query : {}\n".format(trd_ctx.history_deal_list_query(code="", start="", end="2018-6-1")))

    sleep(10)
    trd_ctx.close()


def trade_hk_test():
    '''
    港股交易测试
    :return:
    '''
    trd_ctx = OpenHKTradeContext(host='127.0.0.1', port=11111)
    trd_ctx.set_handler(TradeOrderTest())
    trd_ctx.set_handler(TradeDealTest())
    trd_ctx.start()

    # 交易请求必须先解锁 !!!
    pwd_unlock = '979899'
    print("* unlock_trade : {}\n".format(trd_ctx.unlock_trade(pwd_unlock)))

    # """
    print("* accinfo_query : {}\n".format(trd_ctx.accinfo_query()))
    print("* position_list_query : {}\n".format(trd_ctx.position_list_query(pl_ratio_min=-50, pl_ratio_max=50)))
    print("* order_list_query : {}\n".format(trd_ctx.order_list_query(status_filter_list=[OrderStatus.DISABLED])))
    print("* get_acc_list : {}\n".format(trd_ctx.get_acc_list()))
    print("* order_list_query : {}\n".format(trd_ctx.order_list_query(status_filter_list=[OrderStatus.SUBMITTED])))

    ret_code, ret_data = trd_ctx.place_order(700.0, 100, "HK.00700", TrdSide.SELL)
    print("* place_order : {}\n".format(ret_data))
    if ret_code == RET_OK:
        order_id = ret_data['order_id'][0]
        print("* modify_order : {}\n".format(trd_ctx.modify_order(ModifyOrderOp.CANCEL, order_id, 0, 0)))

    print("* deal_list_query : {}\n".format(trd_ctx.deal_list_query(code="00700")))
    print("* history_order_list_query : {}\n".format(trd_ctx.history_order_list_query(status_filter_list=[OrderStatus.FILLED_ALL, OrderStatus.FILLED_PART],
                                           code="00700", start="", end="2018-2-1")))

    print("* history_deal_list_query : {}\n".format(trd_ctx.history_deal_list_query(code="", start="", end="2018-6-1")))
    # """

    sleep(100000)
    trd_ctx.close()


if __name__ =="__main__":
    set_futu_debug_model(True)
    '''
    默认rsa密钥在futu.common下的conn_key.txt
    注意同步配置FutuOpenD的FTGateway.xml中的 rsa_private_key 字段
    '''
    # SysConfig.set_init_rsa_file()

    ''' 是否启用协议加密 '''
    # SysConfig.enable_proto_encrypt(False)

    '''设置通讯协议格式 '''
    # SysConfig.set_proto_fmt(ProtoFMT.Json)

    '''设置client信息'''
    # SysConfig.set_client_info('sample', 0)

    ''' 行情api测试 '''
    quote_test()

    ''' 交易api测试 '''
    # trade_hk_test()

    # trade_hkcc_test()






# -*- coding: utf-8 -*-

import time
from abc import ABCMeta, abstractmethod
from collections import namedtuple
from time import sleep
# from typing import Optional
from threading import Timer
from datetime import datetime
from threading import RLock, Thread
from futu.common.utils import *
from futu.common.handler_context import HandlerContext
from futu.quote.quote_query import InitConnect, TestCmd
from futu.quote.quote_query import GlobalStateQuery
from futu.quote.quote_query import KeepAlive, parse_head
from futu.common.conn_mng import FutuConnMng
from futu.common.network_manager import NetManager
from .err import Err
from .constant import ContextStatus
from .callback_executor import CallbackExecutor, CallbackItem
from .ft_logger import *

_SyncReqRet = namedtuple('_SyncReqRet', ('ret', 'msg'))


class OpenContextBase(object):
    """Base class for set context"""
    metaclass__ = ABCMeta

    def __init__(self, host, port, is_async_connect, is_encrypt=None):
        self.__host = host
        self.__port = port
        self._callback_executor = CallbackExecutor()
        self._net_mgr = NetManager.default()
        self._handler_ctx = HandlerContext(self._is_proc_run)
        self._lock = RLock()
        self._status = ContextStatus.START
        self._connect_err = None  # rsa加密失败时为Err.RsaErr, 否则为str
        self._proc_run = True
        self._sync_req_ret = None
        self._sync_conn_id = 0
        self._conn_id = 0
        self._keep_alive_interval = 10
        self._last_keep_alive_time = datetime.now()
        self._reconnect_timer = None
        self._reconnect_interval = 8  # 重试连接的间隔
        self._sync_query_connect_timeout = None
        self._keep_alive_fail_count = 0
        self._is_encrypt = is_encrypt
        if self.is_encrypt():
            assert SysConfig.INIT_RSA_FILE != '', Err.NotSetRSAFile.text
        self._net_mgr.start()

        if not is_async_connect:
            self._socket_reconnect_and_wait_ready()

            while True:
                with self._lock:
                    if self._status == ContextStatus.READY or self._status == ContextStatus.CLOSED:
                        break
                sleep(0.02)
        else:
            self._wait_reconnect(0)

    def get_login_user_id(self):
        """
        get login user id
        :return: user id(int64)
        """
        with self._lock:
            return FutuConnMng.get_conn_user_id(self._sync_conn_id)

    def __del__(self):
        self._close()
        
    def __enter__(self):
        pass

    def __exit__(self, t, v, tb):
        self._close()

    @property
    def status(self):
        with self._lock:
            return self._status

    @property
    def connect_err_msg(self):
        with self._lock:
            if self._connect_err is Err.RsaErr:
                return Err.RsaErr.text
            return self._connect_err

    @property
    def connect_err(self):
        return self._connect_err

    def set_sync_query_connect_timeout(self, timeout):
        with self._lock:
            self._sync_query_connect_timeout = timeout

    @property
    def reconnect_interval(self):
        with self._lock:
            return self._reconnect_interval

    @reconnect_interval.setter
    def reconnect_interval(self, value):
        with self._lock:
            self._reconnect_interval = value

    @abstractmethod
    def close(self):
        """
        to call close old obj before loop create new, otherwise socket will encounter error 10053 or more!
        """
        self._close()

    @abstractmethod
    def on_api_socket_reconnected(self):
        """
        callback after reconnect ok
        """
        # logger.debug("on_api_socket_reconnected obj ID={}".format(id(self)))
        return RET_OK, ''

    def _close(self):
        with self._lock:
            if self._status == ContextStatus.CLOSED:
                return
            self._status = ContextStatus.CLOSED
            net_mgr = self._net_mgr
            conn_id = self._conn_id
            self._conn_id = 0
            self._net_mgr = None
            self.stop()
            self._callback_executor.close()
            self._handler_ctx = None
            if self._reconnect_timer is not None:
                self._reconnect_timer.cancel()
                self._reconnect_timer = None
        if conn_id > 0:
            net_mgr.close(conn_id)
        net_mgr.stop()

    def start(self):
        """
        启动异步接收推送数据
        """
        with self._lock:
            self._proc_run = True

    def stop(self):
        """
        停止异步接收推送数据
        """
        with self._lock:
            self._proc_run = False

    def set_handler(self, handler):
        """
        设置异步回调处理对象

        :param handler: 回调处理对象，必须是以下类的子类实例

                    ===============================    =========================
                     类名                                 说明
                    ===============================    =========================
                    StockQuoteHandlerBase               报价处理基类
                    OrderBookHandlerBase                摆盘处理基类
                    CurKlineHandlerBase                 实时k线处理基类
                    TickerHandlerBase                   逐笔处理基类
                    RTDataHandlerBase                   分时数据处理基类
                    BrokerHandlerBase                   经济队列处理基类
                    PriceReminderHandlerBase            到价提醒处理基类
                    ===============================    =========================

        :return: RET_OK: 设置成功

                RET_ERROR: 设置失败
        """
        with self._lock:
            if self._handler_ctx is not None:
                return self._handler_ctx.set_handler(handler)
        return RET_ERROR

    def _is_proc_run(self):
        with self._lock:
            return self._proc_run

    def _get_sync_query_processor(self, pack_func, unpack_func, is_create_socket=True):
        """
        synchronize the query processor
        :param pack_func: back
        :param unpack_func: unpack
        :return: sync_query_processor
        """

        def sync_query_processor(**kargs):
            """sync query processor"""
            start_time = datetime.now()
            while True:
                with self._lock:
                    if self._status == ContextStatus.READY:
                        net_mgr = self._net_mgr
                        conn_id = self._conn_id
                        break
                    elif self._status == ContextStatus.CLOSED:
                        if self.connect_err_msg is not None:
                            return RET_ERROR, self.connect_err_msg, None
                        else:
                            return RET_ERROR, Err.ConnectionClosed.text, None

                if self._sync_query_connect_timeout is not None:
                    elapsed_time = datetime.now() - start_time
                    if elapsed_time.total_seconds() >= self._sync_query_connect_timeout:
                        return RET_ERROR, Err.Timeout.text, None
                sleep(0.01)

            try:
                ret_code, msg, req_str = pack_func(**kargs)
                if ret_code != RET_OK:
                    return ret_code, msg, None

                ret_code, msg, rsp_str = net_mgr.sync_query(conn_id, req_str)
                if ret_code != RET_OK:
                    return ret_code, msg, None

                ret_code, msg, content = unpack_func(rsp_str)
                if ret_code != RET_OK:
                    return ret_code, msg, None
            except Exception as e:
                logger.error(traceback.format_exc())
                return RET_ERROR, str(e), None

            return RET_OK, msg, content

        return sync_query_processor

    def _send_async_req(self, req_str):
        conn_id = 0
        net_mgr = None
        with self._lock:
            if self._status != ContextStatus.READY:
                return RET_ERROR, 'Context closed or not ready'
            conn_id = self._conn_id
            net_mgr = self._net_mgr
        return net_mgr.send(conn_id, req_str)

    def _socket_reconnect_and_wait_ready(self):
        """
        sync_socket & async_socket recreate
        :return: (ret, msg)
        """
        logger.info("Start connecting: host={}; port={};".format(self.__host, self.__port))
        with self._lock:
            self._status = ContextStatus.CONNECTING
            # logger.info("try connecting: host={}; port={};".format(self.__host, self.__port))
            ret, msg, conn_id = self._net_mgr.connect((self.__host, self.__port), self, 5, self.is_encrypt())
            if ret == RET_OK:
                self._conn_id = conn_id
            else:
                logger.warning(msg)

        if ret == RET_OK:
            while True:
                with self._lock:
                    if self._sync_req_ret is not None:
                        if self._sync_req_ret.ret == RET_OK:
                            self._status = ContextStatus.READY
                        else:
                            ret, msg = self._sync_req_ret.ret, self._sync_req_ret.msg
                        self._sync_req_ret = None
                        break
                sleep(0.01)

        if ret == RET_OK:
            ret, msg = self.on_api_socket_reconnected()
        else:
            self._wait_reconnect(self.reconnect_interval)
        return RET_OK, ''

    def get_sync_conn_id(self):
        with self._lock:
            return self._sync_conn_id

    def get_async_conn_id(self):
        return self.get_sync_conn_id()

    def get_global_state(self):
        """
        获取全局状态

        :return: (ret, data)

                ret == RET_OK data为包含全局状态的字典，含义如下

                ret != RET_OK data为错误描述字符串

                =====================   ===========   ==============================================================
                key                      value类型                        说明
                =====================   ===========   ==============================================================
                market_sz               str            深圳市场状态，参见MarketState
                market_us               str            美国市场状态，参见MarketState
                market_sh               str            上海市场状态，参见MarketState
                market_hk               str            香港市场状态，参见MarketState
                market_hkfuture         str            香港期货市场状态，参见MarketState
                market_usfuture         str            美国期货市场状态，参见MarketState
                server_ver              str            FutuOpenD版本号
                trd_logined             str            '1'：已登录交易服务器，'0': 未登录交易服务器
                qot_logined             str            '1'：已登录行情服务器，'0': 未登录行情服务器
                timestamp               str            Futu后台服务器当前时间戳(秒)
                local_timestamp         double         FutuOpenD运行机器当前时间戳(
                =====================   ===========   ==============================================================
        :example:

        .. code:: python

        from futu import *
        quote_ctx = OpenQuoteContext(host='127.0.0.1', port=11111)
        print(quote_ctx.get_global_state())
        quote_ctx.close()
        """
        query_processor = self._get_sync_query_processor(
            GlobalStateQuery.pack_req, GlobalStateQuery.unpack_rsp)

        kargs = {
            'user_id': self.get_login_user_id(),
            'conn_id': self.get_sync_conn_id(),
        }
        ret_code, msg, state_dict = query_processor(**kargs)
        if ret_code != RET_OK:
            return ret_code, msg

        return RET_OK, state_dict

    def is_encrypt(self):
        with self._lock:
            if self._is_encrypt is None:
                return SysConfig.is_proto_encrypt()
            return self._is_encrypt

    def on_connected(self, conn_id):
        logger.info('Connected : conn_id={0}; '.format(conn_id))
        kargs = {
            'client_ver': int(SysConfig.get_client_ver()),
            'client_id': str(SysConfig.get_client_id()),
            'recv_notify': True,
            'is_encrypt': self.is_encrypt(),
            'push_proto_fmt': SysConfig.get_proto_fmt()
        }

        ret, msg, req_str = InitConnect.pack_req(**kargs)
        if ret == RET_OK:
            ret, msg = self._net_mgr.send(conn_id, req_str)
        else:
            logger.warning(FTLog.make_log_msg('InitConnect.pack_req fail', msg=msg))

        if ret != RET_OK:
            with self._lock:
                self._sync_req_ret = _SyncReqRet(RET_ERROR, msg)
                self._connect_err = Err.RsaErr
            self.close()

    def on_error(self, conn_id, err):
        logger.warning('Connect error: conn_id={0}; err={1};'.format(conn_id, err))
        with self._lock:
            if self._status == ContextStatus.CLOSED:
                return
            if self._status != ContextStatus.CONNECTING:
                self._wait_reconnect()
            else:
                self._sync_req_ret = _SyncReqRet(RET_ERROR, str(err))

    def on_closed(self, conn_id):
        logger.warning('Connect closed: conn_id={0}'.format(conn_id))
        with self._lock:
            if self._status == ContextStatus.CLOSED:
                return
            if self._status != ContextStatus.CONNECTING:
                self._wait_reconnect()
            else:
                self._sync_req_ret = _SyncReqRet(RET_ERROR, 'Connection closed')

    def on_connect_timeout(self, conn_id):
        logger.warning('Connect timeout: conn_id={0}'.format(conn_id))
        with self._lock:
            self._sync_req_ret = _SyncReqRet(RET_ERROR, Err.Timeout.text)

    def on_packet(self, conn_id, proto_info, ret_code, msg, rsp_pb):
        with self._lock:
            if self._status == ContextStatus.CLOSED:
                return
        if proto_info.proto_id == ProtoId.InitConnect:
            self._handle_init_connect(conn_id, proto_info.proto_id, ret_code, msg, rsp_pb)
        elif proto_info.proto_id == ProtoId.KeepAlive:
            self._handle_keep_alive(conn_id, proto_info.proto_id, ret_code, msg, rsp_pb)
        elif ret_code == RET_OK:
            item = CallbackItem(self, proto_info.proto_id, rsp_pb)
            self._callback_executor.queue.put(item)

    def on_activate(self, conn_id, now):
        with self._lock:
            if self._status != ContextStatus.READY:
                return
            time_elapsed = now - self._last_keep_alive_time
            if time_elapsed.total_seconds() < self._keep_alive_interval:
                return

            logger.debug("Keepalive: conn_id={};".format(conn_id))
            ret, msg, req = KeepAlive.pack_req(self.get_sync_conn_id())
            if ret != RET_OK:
                logger.warning("KeepAlive.pack_req fail: {0}".format(msg))
                return
            ret, msg = self._net_mgr.send(conn_id, req)
            if ret != RET_OK:
                return

            self._last_keep_alive_time = now

    def packet_callback(self, proto_id, rsp_pb):
        with self._lock:
            if self._status != ContextStatus.READY:
                return

            handler_ctx = self._handler_ctx
        if handler_ctx:
            handler_ctx.recv_func(rsp_pb, proto_id)

    def _handle_init_connect(self, conn_id, proto_info, ret, msg, rsp_pb):
        data = None
        if ret == RET_OK:
            ret, msg, data = InitConnect.unpack_rsp(rsp_pb)

        with self._lock:
            self._sync_req_ret = _SyncReqRet(ret, msg)
            if ret == RET_OK:
                conn_info = copy(data)
                conn_info['is_encrypt'] = self.is_encrypt()
                self._sync_conn_id = conn_info['conn_id']
                self._keep_alive_interval = conn_info['keep_alive_interval'] * 4 / 5
                self._net_mgr.set_conn_info(conn_id, conn_info)
                self._last_keep_alive_time = datetime.now()
                FutuConnMng.add_conn(conn_info)
                logger.info(FTLog.make_log_msg("InitConnect ok", conn_id=conn_id, info=conn_info))
            else:
                logger.warning(FTLog.make_log_msg("InitConnect error", msg=msg))
                self._connect_err = msg
                self.close()

    def _handle_keep_alive(self, conn_id, proto_info, ret_code, msg, rsp_pb):
        should_reconnect = False
        with self._lock:
            if ret_code == RET_OK:
                self._keep_alive_fail_count = 0
            else:
                self._keep_alive_fail_count += 1

            if self._keep_alive_fail_count >= 3:
                logger.warning('Fail to recv KeepAlive for 3 times')
                should_reconnect = True

        if should_reconnect:
            self._wait_reconnect()

    def _wait_reconnect(self, wait_reconnect_interval=8):
        net_mgr = None
        conn_id = 0
        with self._lock:
            if self._status == ContextStatus.CLOSED or self._reconnect_timer is not None:
                return
            logger.info('Wait reconnect in {} seconds: host={}; port={};'.format(wait_reconnect_interval,
                                                                                 self.__host,
                                                                                 self.__port))
            net_mgr = self._net_mgr
            conn_id = self._conn_id

            self._status = ContextStatus.CONNECTING
            self._sync_conn_id = 0
            self._conn_id = 0
            self._keep_alive_fail_count = 0
            self._reconnect_timer = Timer(wait_reconnect_interval, self._reconnect)
            self._sync_req_ret = None
            self._reconnect_timer.start()

        net_mgr.close(conn_id)

    def _reconnect(self):
        with self._lock:
            self._reconnect_timer.cancel()
            self._reconnect_timer = None
            if self._status != ContextStatus.CONNECTING:
                return

        self._socket_reconnect_and_wait_ready()

    def test_cmd(self, cmd, params):
        query_processor = self._get_sync_query_processor(
            TestCmd.pack_req, TestCmd.unpack_rsp)

        kargs = {
            'cmd': cmd,
            'params': params,
        }
        ret_code, msg, state_dict = query_processor(**kargs)
        if ret_code != RET_OK:
            return ret_code, msg

        return RET_OK, state_dict
